@@ -3,7 +3,7 @@ The svgdigitizer suite.
 
 EXAMPLES::
 
-    >>> from .test.cli import invoke
+    >>> from svgdigitizer.test.cli import invoke
     >>> invoke(cli, "--help")  # doctest: +NORMALIZE_WHITESPACE
     Usage: cli [OPTIONS] COMMAND [ARGS]...
       The svgdigitizer suite.
@@ -19,10 +19,10 @@ EXAMPLES::
 # ********************************************************************
 #  This file is part of svgdigitizer.
 #
-#        Copyright (C) 2021 Albert Engstfeld
-#        Copyright (C) 2021 Johannes Hermann
-#        Copyright (C) 2021 Julian Rüth
-#        Copyright (C) 2021 Nicolas Hörmann
+#        Copyright (C) 2021-2022 Albert Engstfeld
+#        Copyright (C)      2021 Johannes Hermann
+#        Copyright (C) 2021-2022 Julian Rüth
+#        Copyright (C)      2021 Nicolas Hörmann
 #
 #  svgdigitizer is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -51,6 +51,14 @@ def cli():
     """
 
 
+# The --skewed flag that is shared by many of the subcommands
+skewed_option = click.option(
+    "--skewed",
+    is_flag=True,
+    help="Detect non-orthogonal skewed axes going through the markers instead of assuming that axes are perfectly horizontal and vertical.",
+)
+
+
 def _outfile(template, suffix=None, outdir=None):
     r"""
     Return a file name for writing.
@@ -61,7 +69,7 @@ def _outfile(template, suffix=None, outdir=None):
 
     EXAMPLES::
 
-        >>> from .test.cli import invoke, TemporaryData
+        >>> from svgdigitizer.test.cli import invoke, TemporaryData
         >>> with TemporaryData("**/xy.svg") as directory:
         ...     outname = _outfile(os.path.join(directory, "xy.svg"), suffix=".csv")
         ...     with open(outname, mode="wb") as csv:
@@ -90,6 +98,30 @@ def _outfile(template, suffix=None, outdir=None):
     return template
 
 
+def _create_svgplot(svg, sampling_interval, skewed):
+    r"""
+    Return an :class:`SVGPlot` as read from the stream `svg`.
+
+    EXAMPLES::
+
+        >>> from svgdigitizer.test.cli import invoke, TemporaryData
+        >>> with TemporaryData("**/xy.svg") as directory:
+        ...     svg = os.path.join(directory, "xy.svg")
+        ...     with open(svg, "rb") as infile:
+        ...         _create_svgplot(infile, sampling_interval=None, skewed=False)
+        <svgdigitizer.svgplot.SVGPlot object at 0x...>
+
+    """
+    from svgdigitizer.svg import SVG
+    from svgdigitizer.svgplot import SVGPlot
+
+    return SVGPlot(
+        SVG(svg),
+        sampling_interval=sampling_interval,
+        algorithm="mark-aligned" if skewed else "axis-aligned",
+    )
+
+
 @click.command()
 @click.option(
     "--sampling-interval",
@@ -97,22 +129,21 @@ def _outfile(template, suffix=None, outdir=None):
     default=None,
     help="Sampling interval on the x-axis.",
 )
+@skewed_option
 @click.argument("svg", type=click.File("rb"))
-def plot(svg, sampling_interval):
+def plot(svg, sampling_interval, skewed):
     r"""
     Display a plot of the data traced in an SVG.
 
     EXAMPLES::
 
-        >>> from .test.cli import invoke, TemporaryData
+        >>> from svgdigitizer.test.cli import invoke, TemporaryData
         >>> with TemporaryData("**/xy.svg") as directory:
         ...     invoke(cli, "plot", os.path.join(directory, "xy.svg"))
 
     """
-    from svgdigitizer.svg import SVG
-    from svgdigitizer.svgplot import SVGPlot
-
-    SVGPlot(SVG(svg), sampling_interval=sampling_interval).plot()
+    svgplot = _create_svgplot(svg, sampling_interval=sampling_interval, skewed=skewed)
+    svgplot.plot()
 
 
 @click.command()
@@ -128,8 +159,9 @@ def plot(svg, sampling_interval):
     default=None,
     help="write output files to this directory",
 )
+@skewed_option
 @click.argument("svg", type=click.Path(exists=True))
-def digitize(svg, sampling_interval, outdir):
+def digitize(svg, sampling_interval, outdir, skewed):
     r"""
     Digitize a plot.
 
@@ -137,16 +169,15 @@ def digitize(svg, sampling_interval, outdir):
 
     EXAMPLES::
 
-        >>> from .test.cli import invoke, TemporaryData
+        >>> from svgdigitizer.test.cli import invoke, TemporaryData
         >>> with TemporaryData("**/xy_rate.svg") as directory:
         ...     invoke(cli, "digitize", os.path.join(directory, "xy_rate.svg"))
 
     """
-    from svgdigitizer.svg import SVG
-    from svgdigitizer.svgplot import SVGPlot
-
     with open(svg, "rb") as infile:
-        svg_plot = SVGPlot(SVG(infile), sampling_interval=sampling_interval)
+        svg_plot = _create_svgplot(
+            infile, sampling_interval=sampling_interval, skewed=skewed
+        )
 
     svg_plot.df.to_csv(_outfile(svg, suffix=".csv", outdir=outdir), index=False)
 
@@ -169,7 +200,8 @@ def digitize(svg, sampling_interval, outdir):
     help="write output files to this directory",
 )
 @click.argument("svg", type=click.Path(exists=True))
-def digitize_cv(svg, sampling_interval, metadata, package, outdir):
+@skewed_option
+def digitize_cv(svg, sampling_interval, metadata, package, outdir, skewed):
     r"""
     Digitize a cylic voltammogram.
 
@@ -177,7 +209,7 @@ def digitize_cv(svg, sampling_interval, metadata, package, outdir):
 
     EXAMPLES::
 
-        >>> from .test.cli import invoke, TemporaryData
+        >>> from svgdigitizer.test.cli import invoke, TemporaryData
         >>> with TemporaryData("**/xy_rate.svg") as directory:
         ...     invoke(cli, "cv", os.path.join(directory, "xy_rate.svg"))
 
@@ -185,7 +217,7 @@ def digitize_cv(svg, sampling_interval, metadata, package, outdir):
 
     The command can be invoked on files in the current directory::
 
-        >>> from .test.cli import invoke, TemporaryData
+        >>> from svgdigitizer.test.cli import invoke, TemporaryData
         >>> cwd = os.getcwd()
         >>> with TemporaryData("**/xy_rate.svg") as directory:
         ...     os.chdir(directory)
@@ -196,7 +228,7 @@ def digitize_cv(svg, sampling_interval, metadata, package, outdir):
 
     The command can be invoked without sampling when data is not given in volts::
 
-        >>> from .test.cli import invoke, TemporaryData
+        >>> from svgdigitizer.test.cli import invoke, TemporaryData
         >>> from svgdigitizer.svg import SVG
         >>> from svgdigitizer.svgplot import SVGPlot
         >>> from svgdigitizer.electrochemistry.cv import CV
@@ -208,13 +240,11 @@ def digitize_cv(svg, sampling_interval, metadata, package, outdir):
 
     """
     from svgdigitizer.electrochemistry.cv import CV
-    from svgdigitizer.svg import SVG
-    from svgdigitizer.svgplot import SVGPlot
 
     if sampling_interval is not None:
         # Rewrite the sampling interval in terms of the unit on the x-axis.
         with open(svg, "rb") as infile:
-            cv = CV(SVGPlot(SVG(infile)))
+            cv = CV(_create_svgplot(infile, sampling_interval=None, skewed=skewed))
 
             from astropy import units as u
 
@@ -227,7 +257,7 @@ def digitize_cv(svg, sampling_interval, metadata, package, outdir):
 
     with open(svg, "rb") as infile:
         cv = CV(
-            SVGPlot(SVG(infile), sampling_interval=sampling_interval),
+            _create_svgplot(infile, sampling_interval=sampling_interval, skewed=skewed),
             metadata=metadata,
         )
 
@@ -235,17 +265,54 @@ def digitize_cv(svg, sampling_interval, metadata, package, outdir):
     cv.df.to_csv(csvname, index=False)
 
     if package:
-        from datapackage import Package
-
-        package = Package(cv.metadata, base_path=outdir or os.path.dirname(csvname))
-        package.infer(os.path.basename(csvname))
+        package = _create_package(cv.metadata, csvname, outdir)
 
     with open(
         _outfile(svg, suffix=".json", outdir=outdir),
         mode="w",
         encoding="utf-8",
     ) as json:
-        _write_metadata(json, package.descriptor if package else cv.metadata)
+        _write_metadata(json, package.to_dict() if package else cv.metadata)
+
+
+def _create_package(metadata, csvname, outdir):
+    r"""
+    Return a data package built from a :param:`metadata` dict and tabular data
+    in :param:`csvname`.
+
+    This is a helper method for :meth:`digitize_cv`.
+    """
+    from frictionless import Package, Resource, Schema
+
+    package = Package(
+        metadata,
+        resources=[
+            Resource(
+                path=os.path.basename(csvname),
+                basepath=outdir or os.path.dirname(csvname),
+            )
+        ],
+    )
+    package.infer()
+    # Update fields in the datapackage describing the data in the CSV
+    package_schema = package["resources"][0]["schema"]
+    data_description_schema = Schema(fields=package["data description"]["fields"])
+
+    new_fields = []
+    for name in package_schema.field_names:
+        if not name in data_description_schema.field_names:
+            raise KeyError(
+                f"Field with name {name} is not specified in `data_descripton.fields`."
+            )
+        new_fields.append(
+            data_description_schema.get_field(name).to_dict()
+            | package_schema.get_field(name).to_dict()
+        )
+
+    package["resources"][0]["schema"]["fields"] = new_fields
+    del package["data description"]["fields"]
+
+    return package
 
 
 def _write_metadata(out, metadata):
@@ -333,7 +400,7 @@ def paginate(onlypng, pdf, outdir):
 
     EXAMPLES::
 
-        >>> from .test.cli import invoke, TemporaryData
+        >>> from svgdigitizer.test.cli import invoke, TemporaryData
         >>> with TemporaryData("**/mustermann_2021_svgdigitizer_1.pdf") as directory:
         ...     invoke(cli, "paginate", os.path.join(directory, "mustermann_2021_svgdigitizer_1.pdf"))
 
